@@ -13,10 +13,11 @@ from cryptofeed.backends.backend import BackendBookCallback, BackendCallback
 
 
 class MongoCallback:
-    def __init__(self, db, host='127.0.0.1', port=27017, key=None, numeric_type=str, **kwargs):
+    def __init__(self, db, host='127.0.0.1', port=27017, key=None, none_to=None, numeric_type=str, **kwargs):
         self.conn = motor.motor_asyncio.AsyncIOMotorClient(host, port)
         self.db = self.conn[db]
         self.numeric_type = numeric_type
+        self.none_to = none_to
         self.collection = key if key else self.default_key
 
     async def write(self, data: dict):
