@@ -29,6 +29,7 @@ DYDX = 'DYDX'
 EXX = 'EXX'
 FTX = 'FTX'
 FTX_US = 'FTX_US'
+FMFW = 'FMFW'
 GATEIO = 'GATEIO'
 GEMINI = 'GEMINI'
 HITBTC = 'HITBTC'
@@ -43,7 +44,6 @@ OKEX = 'OKEX'
 PHEMEX = 'PHEMEX'
 POLONIEX = 'POLONIEX'
 PROBIT = 'PROBIT'
-BITCOINCOM = 'BITCOINCOM'
 UPBIT = 'UPBIT'
 
 
